package edu.swe622.fers.util;

import java.util.Vector;

import org.uddi4j.UDDIException;
import org.uddi4j.datatype.OverviewDoc;
import org.uddi4j.datatype.tmodel.TModel;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.response.TModelDetail;
import org.uddi4j.response.TModelInfo;
import org.uddi4j.response.TModelList;
import org.uddi4j.transport.TransportException;
import org.uddi4j.util.CategoryBag;
import org.uddi4j.util.KeyedReference;

public class ModelUtilities extends Utilities 
{
	/**
	* Locate a technical Model by name
	* @param tModelName the Name
	* @return Vector: This method returns a Vector of tModelKey Strings
	*/
	public static Vector findTModelByName(String tModelName)
	{
		Vector tModelInfoVector = null;
		Vector tModelInfoKeysVector = new Vector();
		String tModelKey = null;
		
		try
		{
			//find model based on Name
			TModelList tModelList =	proxy.find_tModel(tModelName, null, null, null, 0);
			tModelInfoVector = tModelList.getTModelInfos().getTModelInfoVector();
			System.out.println("There are " + tModelList.getTModelInfos().size() + " matching TModels in this registry");	
		
			for(int i = 0; i < tModelInfoVector.size(); i++)
			{
				System.out.println("TModelInfo Key: " + ((TModelInfo)tModelInfoVector.elementAt(i)).getTModelKey());
				tModelInfoKeysVector.addElement(((TModelInfo)tModelInfoVector.elementAt(i)).getTModelKey());
			}
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
		
		if(tModelInfoVector.size() <= 0)
		{
			System.out.println("No Technical Models found for Name: " + tModelName);
		}
		
		return tModelInfoKeysVector;
	} // end findTModelByName
	
	/**
	* Locate a technical Model by key
	* @param tModelName The TModel key used to search for a TModel 
	* @return TModelList: This method returns a TModelList of TModelInfos objects
	*/
	public static TModelList findTModelByKey(String tModelKey)
	{
		KeyedReference reference = new KeyedReference();
		TModelList tModelList = null;
		reference.setTModelKey(tModelKey);
		CategoryBag categoryBag = new CategoryBag();
		categoryBag.add(reference);

		try
		{
			tModelList = proxy.find_tModel(null, categoryBag, null, null, 0); //find model based on Key
		}
		catch(TransportException t)
		{
			System.out.println("Transport Exception: " + t.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
		
		return tModelList;
	} // end findTModelByKey
	
	/**
	* Delete a technical Model by name
	* @param tModelKey TModel name used to delete a TModel
	*/
	public static void deleteTModelByName(String tModelName)
	{
		Vector tModelInfoKeysVector = null;
		
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}
	
			tModelInfoKeysVector = findTModelByName(tModelName);
			
			for(int i = 0; i < tModelInfoKeysVector.size(); i++)
			{
				System.out.println("Preparing to delete TModel key " + tModelInfoKeysVector.elementAt(i).toString());
				DispositionReport report = proxy.delete_tModel(authInfo, tModelInfoKeysVector.elementAt(i).toString());
				
				if(report.success())
				{
					System.out.println("TModel deleted successfully");
				}
				else
				{
					System.out.println("Problem encountered during deleting TModel");
					handleDispositionReport(report);
				}
			} 
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteTModelByName
	
	/**
	* Delete a technical Model by key
	* @param tModelKey TModel key used to delete a TModel
	*/
	public static void deleteTModelByKey(String tModelKey)
	{
		try
		{
			if(authInfo == null)
			{
				authInfo = getAuthorization(); 
			}
	
			System.out.println("Preparing to delete TModel key " + tModelKey);
			DispositionReport report = proxy.delete_tModel(authInfo, tModelKey);
				
			if(report.success())
			{
				System.out.println("TModel deleted successfully");
			}
			else
			{
				System.out.println("Problem encountered during deleting TModel");
				handleDispositionReport(report);
			}
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	} // end deleteTModelByKey
	
	/**
	* Publish a technical Model
	* @param tModelName The TModel name 
	* @param overviewDocString This parameter expects a URL-String representation of the WSDL address.  EX: http://localhost:9080/MyRouter/Test.wsdl
	* @return String: This method returns a String representing the key of the newly published TModel
	*/
	public static String publishTModel(String tModelName, String overviewDocString)
	{
		if(tModelName == null)
		{
			System.out.println("TModel name must not be NULL!");
			return null;
		}
		else
		{
			Vector tModels = new Vector();
			TModel model = new TModel();
			TModelDetail tModelDetail = null;
			
			try
			{
				KeyedReference reference_US = new KeyedReference();
				reference_US.setKeyName("Specification for a Web Service described in WSDL");
				reference_US.setKeyValue("wsdlSpec");
				reference_US.setTModelKey("UUID:C1ACF26D-9672-4404-9D70-39B756E62AB4");
			
				CategoryBag categoryBag = new CategoryBag();
				categoryBag.add(reference_US);
				model.setCategoryBag(categoryBag);
				model.setName(tModelName);
				model.setTModelKey("");
			
				OverviewDoc overviewDoc = new OverviewDoc();
				overviewDoc.setOverviewURL(overviewDocString);
				model.setOverviewDoc(overviewDoc);
				tModels.addElement(model);
			
				if(authInfo == null)
				{
					authInfo = getAuthorization(); 
				}
			
				tModelDetail = proxy.save_tModel(authInfo,tModels);
			
				System.out.println("TModel key: " + ((TModel)tModelDetail.getTModelVector().firstElement()).getTModelKey());
				System.out.println("TModel " + ((TModel)tModelDetail.getTModelVector().firstElement()).getNameString() + " successfully published.");
			}
			catch(TransportException t)
			{
				System.out.println("Transport Exception: " + t.toString());
				return null;
			}
			catch(UDDIException ue)
			{
				System.out.println("UDDI Exception: " + ue.toString());
				return null;
			}
			
			return ((TModel)tModelDetail.getTModelVector().firstElement()).getTModelKey();
		}
	} // end publishtModel
} // end ModelUtilities
