package edu.swe622.fers.util;

import java.net.MalformedURLException;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import org.uddi4j.UDDIException;
import org.uddi4j.client.UDDIProxy;
import org.uddi4j.response.AuthToken;
import org.uddi4j.response.BusinessDetail;
import org.uddi4j.response.DispositionReport;
import org.uddi4j.response.PublisherAssertions;
import org.uddi4j.response.Result;
import org.uddi4j.transport.TransportException;


public class Utilities 
{
	protected static UDDIProxy proxy = null;
	protected static ResourceBundle bundle = null;
	protected static String propertiesFileLocation = "UDDI";
	
	protected static String uddiUserID = null;
	protected static String uddiPassword = null;

	protected BusinessDetail businessDetail = null;
	protected static Map serviceMap = null; //Map of ServiceKey-BusinessKey pairs
	protected static String authInfo = null;
	
	private static void setEnvironment()
	{
		//Set appserver username and password
		System.setProperty("http.basicAuthUserName", getUserID());
		System.setProperty("http.basicAuthPassword", getPassword());
		//System.setProperty("")
		System.setProperty(org.uddi4j.transport.TransportFactory.PROPERTY_NAME, bundle.getString("TransportClassName"));
//		System.setProperty("javax.net.ssl.trustStore", bundle.getString("trustStoreDirPath") + bundle.getString("trustStoreFilename"));
//		System.setProperty("javax.net.ssl.trustStorePassword", bundle.getString("trustStorePassword"));
		
		//Add SSL as a supported protocol.
		//System.setProperty("java.protocol.handler.pkgs",
		System.setProperty("java.protocol.handler.pkgs", "com.ibm.net.ssl.internal.www.protocol");
		System.out.println("Set java.protocol.handler.pkgs");
	}

	protected static String getUserID()
	{
		return uddiUserID;
	}
	
	protected static String getPassword()
	{
		return uddiPassword;
	}
	
	protected static void setAuthorization(String user, String password)
	{
		uddiUserID = user;
		uddiPassword = password;	
	}
	
	protected static String getAuthorization()
	{
		//Get authorization token
		String authInfo = null;
		
		try
		{
			AuthToken authToken = proxy.get_authToken(getUserID(),getPassword());
			authInfo = authToken.getAuthInfoString();
		}
		catch (Exception e)
		{
			System.out.println("An exception was thrown when getting the authorization token:\n"+e);
			System.exit(1);
		}
		
		if(authInfo!=null)
		{
			System.out.println("Successfully obtained authorization token and its info string.");
		}
		
		return authInfo;
	}
	
	public static void getPublisherAssertions()
	{
		try
		{
			PublisherAssertions publisherAssertions = proxy.get_publisherAssertions(getAuthorization());
			System.out.println("Operator: " + publisherAssertions.getOperator());
			System.out.println("Authorized Name: " + publisherAssertions.getAuthorizedName());
		}
		catch(TransportException te)
		{
			System.out.println("Transport Exception: " + te.toString());
		}
		catch(UDDIException ue)
		{
			System.out.println("UDDI Exception: " + ue.toString());
		}
	}
		
	/**
	* Initializes a UDDIProxy object given the credentials passed
	* @param user Username used the interact with the UDDI Registry
	* @param password Password used the interact with the UDDI Registry
	*/
	public static void setupProxy(String user, String password)
	{
		try
		{
			proxy = new UDDIProxy();
			bundle = Configurator.getResourceBundle(propertiesFileLocation);
			
			setAuthorization(bundle.getString("uddiUserID"),bundle.getString("uddiPassword"));
			setEnvironment();
			
			proxy.setInquiryURL(bundle.getString("inquiryURL"));
			proxy.setPublishURL(bundle.getString("publishURL"));
		} 
		catch (MalformedURLException e) 
		{
			System.out.println("MalformedURLException was thrown when attempting to set the UDDI proxy's URLs");
			System.out.println(e.toString());
			System.exit(1);
		}
		catch(Exception ex)
		{
			System.out.println(ex.toString());
		}
	}
	
	//UDDI proxy object representing the UDDI registry.
	protected static UDDIProxy getProxy()
	{
		return proxy;	
	}
	
	protected static void handleDispositionReport(DispositionReport report)
	{
		System.out.println("handleDispositionReport");
		Vector resultVector = null;
		Result result = null;
		String err = null;
		
		/*resultVector = (Vector)report.getResultVector();
		System.out.println("resultVector size: " + resultVector.size());
	
		for(int i = 0; i < resultVector.size(); i++)
		{
			err = ((Result)resultVector.elementAt(i)).getErrno();
			System.out.println("err no" + err);	
		}*/
	}
} // end Utilities
