package guifers;

/**
 * PropertyFilter defines the file filter in the File Chooser.
 * When the file chooser open, we feed this filter into the file chooser
 * to open/show only certain types of files to help the user choosing the 
 * correct type of file in the first selection.
 * 
 * @author Krishna Rudravarjula
 */
import java.io.File;
import javax.swing.filechooser.*;

/* ImageFilter.java is used by FileChooserDemo2.java. */
public class PropertyFilter extends FileFilter {

    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String extension = getExtension(f);
        if (extension != null) {
            if (extension.equals("properties")) {
                    return true;
            } else {
                return false;
            }
        }

        return false;
    }


    public String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
    //The description of this filter
    public String getDescription() {
        return "*.properties";
    }
}
